@@ -176,6 +176,7 @@ class BaseGPTIndex(Generic[IS]):
                     )
                 results.append(sub_index_struct)
             elif isinstance(doc, Document):
+                docstore.set_document_hash(doc.get_doc_id(), doc.get_doc_hash())
                 results.append(doc)
             else:
                 raise ValueError(f"Invalid document type: {type(doc)}.")
@@ -325,6 +326,27 @@ class BaseGPTIndex(Generic[IS]):
         self.delete(document.get_doc_id(), **update_kwargs.pop("delete_kwargs", {}))
         self.insert(document, **update_kwargs.pop("insert_kwargs", {}))
 
+    def refresh(
+        self, documents: List[BaseDocument], **update_kwargs: Any
+    ) -> List[bool]:
+        """Refresh an index with documents that have changed.
+
+        This allows users to save LLM and Embedding model calls, while only
+        updating documents that have any changes in text or extra_info. It
+        will also insert any documents that previously were not stored.
+        """
+        refreshed_documents = [False] * len(documents)
+        for i, document in enumerate(documents):
+            existing_doc_hash = self._docstore.get_document_hash(document.get_doc_id())
+            if existing_doc_hash != document.get_doc_hash():
+                self.update(document, **update_kwargs)
+                refreshed_documents[i] = True
+            elif existing_doc_hash is None:
+                self.insert(document, **update_kwargs.pop("insert_kwargs", {}))
+                refreshed_documents[i] = True
+
+        return refreshed_documents
+
     def _preprocess_query(self, mode: QueryMode, query_kwargs: Dict) -> None:
         """Preprocess query.
 
@@ -394,6 +416,69 @@ class BaseGPTIndex(Generic[IS]):
             )
             return query_runner.query(query_str, self._index_struct)
 
+    async def aquery(
+        self,
+        query_str: Union[str, QueryBundle],
+        mode: str = QueryMode.DEFAULT,
+        query_transform: Optional[BaseQueryTransform] = None,
+        **query_kwargs: Any,
+    ) -> Response:
+        """Asynchronously answer a query.
+
+        When `query` is called, we query the index with the given `mode` and
+        `query_kwargs`. The `mode` determines the type of query to run, and
+        `query_kwargs` are parameters that are specific to the query type.
+
+        For a comprehensive documentation of available `mode` and `query_kwargs` to
+        query a given index, please visit :ref:`Ref-Query`.
+
+
+        """
+        # TODO: currently we don't have async versions of all
+        # underlying functions. Setting use_async=True
+        # will cause async nesting errors because we assume
+        # it's called in a synchronous setting.
+        use_async = False
+
+        mode_enum = QueryMode(mode)
+        if mode_enum == QueryMode.RECURSIVE:
+            # TODO: deprecated, use ComposableGraph instead.
+            if "query_configs" not in query_kwargs:
+                raise ValueError("query_configs must be provided for recursive mode.")
+            query_configs = query_kwargs["query_configs"]
+            query_runner = QueryRunner(
+                self._llm_predictor,
+                self._prompt_helper,
+                self._embed_model,
+                self._docstore,
+                self._index_registry,
+                query_configs=query_configs,
+                query_transform=query_transform,
+                recursive=True,
+                use_async=use_async,
+            )
+            return await query_runner.aquery(query_str, self._index_struct)
+        else:
+            self._preprocess_query(mode_enum, query_kwargs)
+            # TODO: pass in query config directly
+            query_config = QueryConfig(
+                index_struct_type=self._index_struct.get_type(),
+                query_mode=mode_enum,
+                query_kwargs=query_kwargs,
+            )
+            query_runner = QueryRunner(
+                self._llm_predictor,
+                self._prompt_helper,
+                self._embed_model,
+                self._docstore,
+                self._index_registry,
+                query_configs=[query_config],
+                query_transform=query_transform,
+                recursive=False,
+                use_async=use_async,
+            )
+            return await query_runner.aquery(query_str, self._index_struct)
+
     @classmethod
     @abstractmethod
     def get_query_map(cls) -> Dict[str, Type[BaseGPTIndexQuery]]:
