@@ -20,10 +20,9 @@ from gpt_index.indices.response.builder import (
     TextChunk,
 )
 from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
-from gpt_index.prompts.default_prompts import (
-    DEFAULT_REFINE_PROMPT,
-    DEFAULT_TEXT_QA_PROMPT,
-)
+from gpt_index.optimization.optimizer import BaseTokenUsageOptimizer
+from gpt_index.prompts.default_prompt_selectors import DEFAULT_REFINE_PROMPT_SEL
+from gpt_index.prompts.default_prompts import DEFAULT_TEXT_QA_PROMPT
 from gpt_index.prompts.prompts import QuestionAnswerPrompt, RefinePrompt
 from gpt_index.response.schema import RESPONSE_TYPE, Response, StreamingResponse
 from gpt_index.token_counter.token_counter import llm_token_counter
@@ -95,6 +94,7 @@ class BaseGPTIndexQuery(Generic[IS]):
         recursive: bool = False,
         streaming: bool = False,
         doc_ids: Optional[List[str]] = None,
+        optimizer: Optional[BaseTokenUsageOptimizer] = None,
     ) -> None:
         """Initialize with parameters."""
         if index_struct is None:
@@ -116,7 +116,7 @@ class BaseGPTIndexQuery(Generic[IS]):
         self._response_mode = ResponseMode(response_mode)
 
         self.text_qa_template = text_qa_template or DEFAULT_TEXT_QA_PROMPT
-        self.refine_template = refine_template or DEFAULT_REFINE_PROMPT
+        self.refine_template = refine_template or DEFAULT_REFINE_PROMPT_SEL
         self._include_summary = include_summary
 
         self._response_kwargs = response_kwargs or {}
@@ -134,6 +134,7 @@ class BaseGPTIndexQuery(Generic[IS]):
         self._recursive = recursive
         self._streaming = streaming
         self._doc_ids = doc_ids
+        self._optimizer = optimizer
 
     def _should_use_node(
         self, node: Node, similarity_tracker: Optional[SimilarityTracker] = None
@@ -222,6 +223,15 @@ class BaseGPTIndexQuery(Generic[IS]):
         )
         return response
 
+    async def _agive_response_for_nodes(self, query_str: str) -> RESPONSE_TEXT_TYPE:
+        """Give response for nodes."""
+        response = await self.response_builder.aget_response(
+            query_str,
+            mode=self._response_mode,
+            **self._response_kwargs,
+        )
+        return response
+
     def get_nodes_and_similarities_for_response(
         self, query_bundle: QueryBundle
     ) -> List[Tuple[Node, Optional[float]]]:
@@ -257,26 +267,34 @@ class BaseGPTIndexQuery(Generic[IS]):
         """Get extra info for response."""
         return None
 
-    def _query(self, query_bundle: QueryBundle) -> RESPONSE_TYPE:
-        """Answer a query."""
-        self.response_builder.reset()
-        # TODO: remove _query and just use query
-        tuples = self.get_nodes_and_similarities_for_response(query_bundle)
-
+    def _prepare_response_builder(
+        self,
+        response_builder: ResponseBuilder,
+        query_bundle: QueryBundle,
+        tuples: List[Tuple[Node, Optional[float]]],
+    ) -> None:
+        """Prepare response builder and return values for query time."""
+        response_builder.reset()
         for node, similarity in tuples:
             text, response = self._get_text_from_node(query_bundle, node)
-            self.response_builder.add_node_as_source(node, similarity=similarity)
+            response_builder.add_node_as_source(node, similarity=similarity)
             if response is not None:
                 # these are source nodes from within this node (when it's an index)
                 for source_node in response.source_nodes:
-                    self.response_builder.add_source_node(source_node)
-            self.response_builder.add_text_chunks([text])
-
-        if self._response_mode != ResponseMode.NO_TEXT:
-            response_str = self._give_response_for_nodes(query_bundle.query_str)
-        else:
-            response_str = None
+                    response_builder.add_source_node(source_node)
+            if self._optimizer is not None:
+                response_builder.add_text_chunks(
+                    [TextChunk(text=self._optimizer.optimize(query_bundle, text.text))]
+                )
+            else:
+                response_builder.add_text_chunks([text])
 
+    def _prepare_response_output(
+        self,
+        response_str: Optional[RESPONSE_TEXT_TYPE],
+        tuples: List[Tuple[Node, Optional[float]]],
+    ) -> RESPONSE_TYPE:
+        """Prepare response object from response string."""
         response_extra_info = self._get_extra_info_for_response(
             [node for node, _ in tuples]
         )
@@ -296,6 +314,36 @@ class BaseGPTIndexQuery(Generic[IS]):
         else:
             raise ValueError("Response must be a string or a generator.")
 
+    def _query(self, query_bundle: QueryBundle) -> RESPONSE_TYPE:
+        """Answer a query."""
+        # TODO: remove _query and just use query
+        tuples = self.get_nodes_and_similarities_for_response(query_bundle)
+
+        # prepare response builder
+        self._prepare_response_builder(self.response_builder, query_bundle, tuples)
+
+        if self._response_mode != ResponseMode.NO_TEXT:
+            response_str = self._give_response_for_nodes(query_bundle.query_str)
+        else:
+            response_str = None
+
+        return self._prepare_response_output(response_str, tuples)
+
+    async def _aquery(self, query_bundle: QueryBundle) -> RESPONSE_TYPE:
+        """Answer a query asynchronously."""
+        # TODO: remove _query and just use query
+        tuples = self.get_nodes_and_similarities_for_response(query_bundle)
+
+        # prepare response builder
+        self._prepare_response_builder(self.response_builder, query_bundle, tuples)
+
+        if self._response_mode != ResponseMode.NO_TEXT:
+            response_str = await self._agive_response_for_nodes(query_bundle.query_str)
+        else:
+            response_str = None
+
+        return self._prepare_response_output(response_str, tuples)
+
     @llm_token_counter("query")
     def query(self, query_bundle: QueryBundle) -> RESPONSE_TYPE:
         """Answer a query."""
@@ -331,3 +379,39 @@ class BaseGPTIndexQuery(Generic[IS]):
                 raise ValueError("Response must be a string or a generator.")
 
         return response
+
+    @llm_token_counter("query")
+    async def aquery(self, query_bundle: QueryBundle) -> RESPONSE_TYPE:
+        """Answer a query."""
+        response = await self._aquery(query_bundle)
+        # if include_summary is True, then include summary text in answer
+        # summary text is set through `set_text` on the underlying index.
+        # TODO: refactor response builder to be in the __init__
+        if self._response_mode != ResponseMode.NO_TEXT and self._include_summary:
+            response_builder = ResponseBuilder(
+                self._prompt_helper,
+                self._llm_predictor,
+                self.text_qa_template,
+                self.refine_template,
+                texts=[TextChunk(self._index_struct.get_text())],
+                streaming=self._streaming,
+            )
+            if isinstance(response, Response):
+                # NOTE: use create and refine for now (default response mode)
+                response_str = await response_builder.aget_response(
+                    query_bundle.query_str,
+                    mode=self._response_mode,
+                    prev_response=response.response,
+                )
+                response.response = cast(str, response_str)
+            elif isinstance(response, StreamingResponse):
+                response_gen = await response_builder.aget_response(
+                    query_bundle.query_str,
+                    mode=self._response_mode,
+                    prev_response=str(response.response_gen),
+                )
+                response.response_gen = cast(Generator, response_gen)
+            else:
+                raise ValueError("Response must be a string or a generator.")
+
+        return response
