@@ -4,6 +4,7 @@ from typing import List, Optional
 from gpt_index.data_structs.data_structs import IndexList, Node
 from gpt_index.indices.query.base import BaseGPTIndexQuery
 from gpt_index.indices.query.embedding_utils import SimilarityTracker
+from gpt_index.indices.query.schema import QueryBundle
 
 
 class BaseGPTListIndexQuery(BaseGPTIndexQuery[IndexList]):
@@ -38,8 +39,7 @@ class GPTListIndexQuery(BaseGPTListIndexQuery):
 
     def _get_nodes_for_response(
         self,
-        query_str: str,
-        verbose: bool = False,
+        query_bundle: QueryBundle,
         similarity_tracker: Optional[SimilarityTracker] = None,
     ) -> List[Node]:
         """Get nodes for response."""
