@@ -6,7 +6,7 @@ from gpt_index.data_structs.data_structs import IndexStruct
 from gpt_index.docstore import DocumentStore
 from gpt_index.embeddings.base import BaseEmbedding
 from gpt_index.indices.prompt_helper import PromptHelper
-from gpt_index.indices.query.base import BaseQueryRunner
+from gpt_index.indices.query.base import BaseGPTIndexQuery, BaseQueryRunner
 from gpt_index.indices.query.query_transform import BaseQueryTransform
 from gpt_index.indices.query.schema import QueryBundle, QueryConfig, QueryMode
 from gpt_index.indices.registry import IndexRegistry
@@ -79,19 +79,11 @@ class QueryRunner(BaseQueryRunner):
             query_kwargs["embed_model"] = self._embed_model
         return query_kwargs
 
-    def query(
+    def _get_query_obj(
         self,
-        query_str_or_bundle: Union[str, QueryBundle],
         index_struct: IndexStruct,
-    ) -> Response:
-        """Run query."""
-        # NOTE: Currently, query transform is only run once
-        # TODO: Consider refactor to support index-specific query transform
-        if isinstance(query_str_or_bundle, str):
-            query_bundle = self._query_transform(query_str_or_bundle)
-        else:
-            query_bundle = query_str_or_bundle
-
+    ) -> BaseGPTIndexQuery:
+        """Get query object."""
         index_struct_id = index_struct.get_doc_id()
         index_struct_type = index_struct.get_type()
         if index_struct_id in self._id_to_config_dict:
@@ -117,4 +109,34 @@ class QueryRunner(BaseQueryRunner):
             use_async=self._use_async,
         )
 
+        return query_obj
+
+    def query(
+        self,
+        query_str_or_bundle: Union[str, QueryBundle],
+        index_struct: IndexStruct,
+    ) -> Response:
+        """Run query."""
+        # NOTE: Currently, query transform is only run once
+        # TODO: Consider refactor to support index-specific query transform
+        if isinstance(query_str_or_bundle, str):
+            query_bundle = self._query_transform(query_str_or_bundle)
+        else:
+            query_bundle = query_str_or_bundle
+        query_obj = self._get_query_obj(index_struct)
         return query_obj.query(query_bundle)
+
+    async def aquery(
+        self,
+        query_str_or_bundle: Union[str, QueryBundle],
+        index_struct: IndexStruct,
+    ) -> Response:
+        """Run query."""
+        # NOTE: Currently, query transform is only run once
+        # TODO: Consider refactor to support index-specific query transform
+        if isinstance(query_str_or_bundle, str):
+            query_bundle = self._query_transform(query_str_or_bundle)
+        else:
+            query_bundle = query_str_or_bundle
+        query_obj = self._get_query_obj(index_struct)
+        return await query_obj.aquery(query_bundle)
