@@ -71,6 +71,11 @@ def _mock_kg_triplet_extract(prompt_args: Dict) -> str:
     return prompt_args["text"]
 
 
+def _mock_input(prompt_args: Dict) -> str:
+    """Mock input prompt."""
+    return prompt_args["query_str"]
+
+
 def mock_llmpredictor_predict(prompt: Prompt, **prompt_args: Any) -> Tuple[str, str]:
     """Mock predict method of LLMPredictor.
 
@@ -99,12 +104,21 @@ def mock_llmpredictor_predict(prompt: Prompt, **prompt_args: Any) -> Tuple[str,
         response = _mock_text_to_sql(full_prompt_args)
     elif prompt.prompt_type == PromptType.KNOWLEDGE_TRIPLET_EXTRACT:
         response = _mock_kg_triplet_extract(full_prompt_args)
+    elif prompt.prompt_type == PromptType.SIMPLE_INPUT:
+        response = _mock_input(full_prompt_args)
     else:
         raise ValueError("Invalid prompt to use with mocks.")
 
     return response, formatted_prompt
 
 
+async def mock_llmpredictor_apredict(
+    prompt: Prompt, **prompt_args: Any
+) -> Tuple[str, str]:
+    """Mock apredict method of LLMPredictor."""
+    return mock_llmpredictor_predict(prompt, **prompt_args)
+
+
 def mock_llmchain_predict(**full_prompt_args: Any) -> str:
     """Mock LLMChain predict with a generic response."""
     return "generic response from LLMChain.predict()"
